/*
   a grafix subroutines for ZXASM demo
   Copyright(c) by SCREW/CM/OHG

   Disturb: create a dynamic graphic FX
*/

#define lines 24         // (16) lines in a frame
#define screen_start 0    // (32) pixels offset from up frame border
                          //      must be aligned on 8 (charset)

// SINC macros
// move screen addres in [arg1] on 1 pixel by down
SINC macro arg1
      inc hi(arg1)
      ld a,hi(arg1)
      and 7
      jr nz,@exit    // WARNING: thic local label may be call
                     // 'label already exists' error.
      ld a,lo(arg1)
      add a,32
      ld lo(arg1),a
      jr c,@exit    // WARNING: thic local label may be call
                     // 'label already exists' error.
      ld a,hi(arg1)
      sub 8
      ld hi(arg1),a
@exit:
    endm

/****************************************************************************
                           CLEAR SCREEN
****************************************************************************/
clear_screen:
     ld hl,49152
     ld de,49153
     ld bc,6144
     ld (hl),l
     ldir
     ld bc,767
     ld (hl),6
     ldir
     ret

/***************************************************************************
                  MAKE TABLE WITH SCREEN ADRESSES
****************************************************************************/
make_table:
     ld de,49152
     ld hl,sc_table  // !!! must be ALIGNed on 256 !!!
     ld b,192
@rept:
     push de
     ld a,e
     add a,32
     ld e,a
     ld a,d
     adc a,0
     ld d,a

     ld (hl),e
     inc h
     ld (hl),d
     dec h
     inc l
     pop de
     SINC de
     djnz @rept
     ret

/***************************************************************************
                       DRAW ONE B/W FRAME
****************************************************************************/
// WARNING: stack using !!!
screen_draw:
     ld hl,0
     add hl,sp
     ld (@rest_stack+1),hl

     ld de,offs_buffer
     ld hl,sc_table + screen_start
     ld c,lines
     ld (@rept2+1),hl
@rept1:
     ld (@rest_de+1),de
     ld a,(de)
     and 15
     add a,a
     ld l,a
     ld h,chunk_table / 256 // !!! must be ALIGNed on 256 !!!
     ld e,(hl)
     inc l
     ld d,(hl)

     ld b,8

@rept2:    ld hl,0
           inc l
           ld (@rept2+1),hl
           dec l

           ld a,(hl)
           inc h
           ld h,(hl)
           ld l,a
           ld sp,hl   // make SP=scr_address

     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de
     push de

     djnz @rept2

@rest_de: ld de,0
     inc de
     dec c
     jr nz,@rept1

@rest_stack:
     ld sp,0
     ret

/***************************************************************************
                          DRAW ATTRIBUTES MAP
****************************************************************************/
attr_draw:
     ld hl,at_buffer
     ld de,screen_start/8*32 +49152+6144
     ld ix,offs_buffer
     ld a,lines
@rept1:
     ex af,af'
     ld a,(ix)
     inc ix
     rrca
     rrca
     rrca
     and 31
     neg
     add a,32
     push af
     push hl
     ld b,0
     ld c,a
     add hl,bc
     neg
     add a,32
     jr z,@skip
     ld c,a
     ldir
@skip:
     pop hl
     pop af
     push hl
     jr z,@skip2
     ld c,a
     ldir
@skip2:
     pop hl
     ld bc,32
     add hl,bc
     ex af,af'
     dec a
     jr nz,@rept1
     ret


/***************************************************************************
                             SCROLL IMAGE
****************************************************************************/
// a sinusible scroll
frame_scroll:
         ld hl,offs_buffer
         ld b,lines
@sinpos: ld a,0
         add a,3
         ld (@sinpos+1),a
         ld d,sinus / 256   // !!! must be ALINGed on 256 !!!
         ld e,a
@rept1:
         ld a,(de)
         ld (hl),a
         inc hl
         ld a,e
         add a,7
         ld e,a
         djnz @rept1
         ret


/***************************************************************************
                             CREATE FRAME
****************************************************************************/
frame_create:
// create chunks
          ld hl,chunk_table // !!! must be ALINGed on 256 !!!
          ld de,255   // 1111 1111=e 0000 0000=d
          ld b,16
@rept1:
          ld (hl),e
          inc l
          ld (hl),d
          inc l
          rr e
          rr d
          rl e
          rrc e
          djnz @rept1

// create attributes map
          ld hl,at_buffer
          ld bc,32*lines
          push hl
@at_create:
          get_RND
          ld a,c
          and 7
          ld (hl),a
          inc hl
          dec bc
          ld a,b
          or c
          jr nz,@at_create
          pop hl
          ld c,lines
@at_adjust:
          push hl
          pop ix
          ld b,16
@rept2:
          inc hl
          rlc (hl)
          rlc (hl)
          rlc (hl)
          inc hl
          djnz @rept2

          ld b,16
          ld a,(ix+31)
@rept3:
          ld e,(ix)     // ink at current
          or (ix)
          ld (ix),a
          ld d,(ix+1)   // paper at next
          ld a,d
          or e
          ld (ix+1),a
          inc ix
          inc ix
          ld a,d
          djnz @rept3

          dec c
          jr nz,@at_adjust
          ret

// some RAND function. return random number in [A]
get_RND:
         push ix
         ld ix,@rand
         rlc (ix)
         rrc (ix+1)
         ld a,(ix+2)
         sub (ix)
         ld (ix+2),a
         add a,(ix+1)
         ld (ix+1),a
         pop ix
         ret
@rand:   db 171,147,53


/***************************************************************************
                           DATA ALLOCATION
****************************************************************************/
// buffers for GRAFIX.INC


// +++++ ALIGNED VARIABLES +++++++++++++++++++++++++++++++++++++++++++++
     ALIGN 256
sinus: #BIN_LOAD sinus.dat  // including sintable
     ALIGN 256
sc_table:  ds 512  // this is table for screen addresses
chunk_table: ds 2*16 // 16th 16-bits chunks

// ++++++ UNALIGNED VARIABLES +++++++++++++++++++++++++++++++++++++++++++
at_buffer: ds lines*32  // attrs data
offs_buffer: ds lines  // offsets in a frame
